require 'json'

module FirestoreODM
  class Document
    attr_accessor :document
    attr_reader :data, :changes

    # Constructor.
    # @param document [Google::Cloud::Firestore::DocumentReference] the document reference.
    def initialize document = nil
      unless document.nil?
        @document = document
        @data     = document.get.data
      end

      discard_changes
    end

    # Gets the document id.
    # @return [String] the document id.
    def id
      document.document_id
    end

    # Gets the document path
    # @return [String] the document path.
    def path
      document.document_path
    end

    # Saves any changes made to the document.
    def save
      document.update changes
      @data = document.get.data
      discard_changes
      return
    end

    # Discards any changes made to the document.
    def discard_changes
      @changes = {}
      return
    end

    # Deletes the document.
    def delete
      document.delete
    end

    # Gets a field.
    # @param key [String, Symbol] the field's name.
    # @return the field's value.
    def get key
      changes[key] or data[key]
    end

    # Sets a field.
    # @param key [String, Symbol] the field's name.
    # @param value the field's value.
    def set key, value
      changes[key] = value
      return
    end

    # Converts the document to a JSON string.
    # @param opts options to be passed to {JSON#generate}.
    # @return [String] a JSON string.
    def to_json opts = nil
      data.merge(changes).to_json(opts)
    end

    alias :[] :get
    alias :[]= :set
  end
end